/*
 *     Copyright CERN, Geneva 1989 - Copyright and any other
 *     appropriate legal protection of these computer programs
 *     and associated documentation reserved in all countries
 *     of the world.
 */

/*
 *  Copyright (c) 1984 Marvin Solomon; all rights reserved
 */
/*
 * Various translation tables and constants for describing 327x IBM terminals
 *
 *
 */

#define EBCDICeor                       0x1e

/* translation table to convert from a six bit value to a printable character
 * (used in 12-bit cursor addresses)
 */
extern  unsigned char   SIXBITtoEBCDIC[];

#define byte1(n)        SIXBITtoEBCDIC[((n)>>6) & 077]
#define byte2(n)        SIXBITtoEBCDIC[(n) & 077]

/* values for the AID byte */
#define ENTERkey                        0x7d
#define NOattentionKEY                  0x60

/* 3270 orders */
#define SFchar                          0x1D    /* Start Field */
#define SFEchar                         0x29    /* Start Field Extended */
#define SBAchar                         0x11    /* Set Buffer Address */
#define SAchar                          0x28    /* Set Attribute */
#define MFchar                          0x2C    /* Modify Field */
#define ICchar                          0x13    /* Insert Cursor */
#define PTchar                          0x05    /* Program Tab */
#define RAchar                          0x3C    /* Repeat to Address */
#define EUAchar                         0x12    /* Erase Unprotected to Address */
#define GEchar                          0x08    /* Graphics Escape. */

#define TRANSP_WRITE                    0x70    /* Transparent write (e.g. Kermit) */

/* 3270 commands */
#define WRITEcommand                     1
#define READBUFFERcommand                2
#define READMODIFIEDcommand              6
#define READMOD_ALLcommand              14
#define ERASEWRITEcommand                5
#define ERASEWRITEALTERNATEcommand      13
#define ERASEALLUNPROTECTEDcommand      15
#define WSFcommand                      17


/* bits in the WCC byte */
#define WCCreset                        0x40    /* reset terminal state */
#define WCCbell                         0x04    /* ring bell at end of operation */
#define WCCprinter                      0x08    /* start the printer operation */
#define WCCunlock                       0x02    /* unlock keyboard */
#define WCCresetMDT                     0x01    /* reset all modified-data tags */

/* bits in a start-field byte */
#define SFspecialBITS                   0x40    /* Blinking field (non-standard!!!) */
#define SFmodentryBITS                  0x02    /* Copy of entry SFmodifiedBITS */
#define SFprotectedBITS                 0x20    /* The operator can not change this field */
#define SFnumericBITS                   0x10    /* Numeric field (NB: (prot&num)==autoskip) */
#define SFvisibleBITS                   0x0C    /* Visibility attributes; one of:  */
#define SFinvisibleBITS                 0x0C    /*    do not show the field */
#define SFstandoutBITS                  0x08    /*    show in high intensity */
#define SFnormalBITS                    0x00    /*    show normally */
#define SFmodifiedBITS                  0x01    /* The user has changed this field. */

#define ATTRMASK                        (0x3f & ~SFmodentryBITS)

/* Turn the two bytes p[0] and p[1] into an address.
 * Currently we only support 12-bit encoding.
 */
#define decodepos(p) (((*(p) & 077) << 6) | (*((p)+1) & 077))

/* convert to row/column (mostly for debugging) */
#define ROWCOL(n) ((n) / COLS + 1) , ((n) % COLS + 1)

/*
 * keyboard actions
 * For each action there is a name, an action routine, an argument
 * for this routine and an indicator to say if the action can be
 * taken immediately or must wait until the IBM has unlocked input.
 * An argument of 999 is always used for unimplemented actions.
 * The indicator values are:-
 *      -1      For immediate action (nothing to go to IBM!)
 *       0      Wait until IBM unlocks (then action to IBM)
 *       1      Wait until IBM unlocks (but nothing to go to IBM)
 */

struct  action_entry {
			char    *name;
			void    (*action)();
			short   arg;
			short   immediate;
			char    *act_hmess;
		     };

extern  struct  action_entry    actions[];
